--  ports32.e 
--  by Eric D. Larson
--  I added support for 32-bit I/O
--  This code is derived from "ports.e"
--  by Jacques Deschenes, Baie-Comeau, PQ, Canada, e-mail: desja@quebectel.com
--
--  the functions defined in this file are for reading and writing to I/O ports.
--  *****     Exported functions and procedures ***
--      inp32() to read a double word from port
--      outp32() to write a double word to port
 

-- include machine.e
-- without type_check

include ports.e

sequence outp32Code, inp32Code

outp32Code = {
		#50,        -- PUSH EAX
		#52,        -- PUSH EDX
		#BA,0,0,0,0,-- MOV EDX, PORT  (port, to be poked in) (3) 
		#B8,0,0,0,0,-- MOV EAX, dword  (to be poked in) (8) 
		#EF,        -- OUT DX, EAX    (no #66 prefix)
		#5A,        -- POP EDX
		#58,        -- POP EAX
		#C3         -- RET
	    }

inp32Code = {
		#50,        -- PUSH EAX
		#53,        -- PUSH EBX
		#52,        -- PUSH EDX
		#BA,0,0,0,0,-- MOV EDX, PORT  (port to be poked in) (4)
		#BB,0,0,0,0,-- MOV EBX, storage address (9)
		#ED,        -- IN EAX, DX     (no #66 prefix)
		#89,#03,    -- MOV [EBX],EAX  (no #66 prefix)
		#5A,        -- POP EDX
		#5B,        -- POP EBX
		#58,        -- POP EAX
		#C3,        -- RET
		#0,#0,#0,#0 -- DD ? data to be returned
	   }

atom inp32Asm, inp32Port, inp32Data,
     outp32Asm, outp32Port, outp32Data


function allocate_v(integer count) -- allocate and validate
atom pointer
    pointer = allocate(count)
    if not pointer then
	puts(1,"Fatal error! memory allocation failed in ports.e\n")
	abort(1)
    end if
    return pointer
end function

procedure InitPorts()
-- init code space for assembler routines
integer CodeLength
    
    CodeLength = length(inp32Code)
    inp32Asm = allocate_v(CodeLength)
    poke(inp32Asm,inp32Code)
    poke4(inp32Asm+9,inp32Asm+CodeLength-2) --JD990318 corrected bug reported by Jeff Gokingco
 
    inp32Port = inp32Asm + 4
    inp32Data = inp32Asm+CodeLength-2
    
    outp32Asm = allocate_v(length(outp32Code))
    poke(outp32Asm,outp32Code)
    outp32Port = outp32Asm + 3
    outp32Data = outp32Asm + 8
end procedure -- InitPorts()


global function inp32(word port)
sequence bytes
  poke4(inp32Port,port)
  call(inp32Asm)
  bytes = peek({inp32Data,4})
  return (bytes[4]*16777216)+(bytes[3]*65536)+(bytes[2]*256)+bytes[1]
end function

global procedure outp32(word port, atom w)
  poke4(outp32Port,port)
  poke4(outp32Data,w)
  call(outp32Asm)
end procedure 

InitPorts()


